//-------+---------+---------+---------+---------+---------+---------+-----
// wingcd.c - Multi-Media Wingman Ver01.01 Eternal - Wingman Audio CD Player
//-------+---------+---------+---------+---------+---------+---------+-----

///////////////////////////////////////////////////////////////////////////
// Copyright (C) GENERAL SCIENCE (M.Kasahara) since 1999.01
// Programmed by GENERAL SCIENCE (M.Kasahara)
// Ver00.20 2000.09 - Wingman Ver01.01
///////////////////////////////////////////////////////////////////////////
// !!! Information !!! - Find ">>>" for find function in this source.
///////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <WGM/mwingman.h>

#include "resource.h"

#define TRUE                 1
#define FALSE                0
#define ID_CDAUDIO_RESTART   100
#define ID_CDAUDIO_STOP      101
#define ID_CDAUDIO_PAUSE     102
#define ID_CDAUDIO_RESUME    103
#define ID_CDAUDIO_VOLUME    104
#define ID_CDAUDIO_EJECT     105
#define ID_CDAUDIO_CLOSE     106
#define ID_CDAUDIO_LOCK      107
#define ID_CDAUDIO_UNLOCK    108
#define ID_CDAUDIO_RESET     109
#define ID_CDAUDIO_CATALOGID 110

static char g_cCommand [32];
static int g_nCommand = 0;
static int g_nIndex = -1;
static int g_nVolume = 127;

//-------+---------+---------+---------+---------+---------+---------+-----
// Define Local Prototype :
//-------+---------+---------+---------+---------+---------+---------+-----
static void arginit (int argc, char *argv []);
static void usage (int nUsage);
static int OutCDTrack (int nDevice, wgmCDTrack *pCDTrack);

//-------+---------+---------+---------+---------+---------+---------+-----
// Main function :
//-------+---------+---------+---------+---------+---------+---------+-----
// Function List >>> main :
//               >>> arginit :
//               >>> usage :
//               >>> OutCDTrack :

///////////////////////////////////////////////////////////////////////////
//>>> main :
///////////////////////////////////////////////////////////////////////////
int main (int argc, char *argv [])
{
  wgmCDTrack *pCDTrack;
  int i, nDevice;
  int nStatus = 0;

  arginit (argc, argv);
  usage (FALSE);

  if ((nDevice = wgmOpenDevice_CDAudio ()) < 0)
    exit (1);

  nStatus = wgmInit_CDAudio (nDevice);
  if (nStatus < 0)
  {
    wgmCloseDevice_CDAudio (nDevice);
    exit (1);
  }

  pCDTrack = wgmGetCDTrack_CDAudio (nDevice);
  if (pCDTrack == NULL)
  {
    wgmCloseDevice_CDAudio (nDevice);
    exit (1);
  }

  switch (g_nCommand){
  case ID_CDAUDIO_RESTART:
    wgmPlay_CDAudio (nDevice, pCDTrack->pIndex [0], 
      pCDTrack->pIndex [pCDTrack->nTracks]);
    break;
  case ID_CDAUDIO_CATALOGID:
  case ID_CDAUDIO_STOP:
    wgmStop_CDAudio (nDevice);
    break;
  case ID_CDAUDIO_PAUSE:
    wgmPause_CDAudio (nDevice);
    break;
  case ID_CDAUDIO_RESUME:
    wgmResume_CDAudio (nDevice);
    break;
  case ID_CDAUDIO_EJECT:
    wgmEjectDisk_CDAudio (nDevice);
    break;
  case ID_CDAUDIO_CLOSE:
    wgmCloseDisk_CDAudio (nDevice);
    wgmPlay_CDAudio (nDevice, pCDTrack->pIndex [0], 
      pCDTrack->pIndex [pCDTrack->nTracks]);
    break;
  case ID_CDAUDIO_LOCK:
    wgmLock_CDAudio (nDevice, TRUE);
    break;
  case ID_CDAUDIO_UNLOCK:
    wgmLock_CDAudio (nDevice, FALSE);
    break;
  case ID_CDAUDIO_RESET:
    wgmReset_CDAudio (nDevice);
    break;
  case ID_CDAUDIO_VOLUME:
    wgmSetVolume_CDAudio (nDevice, 0, g_nVolume);
    wgmSetVolume_CDAudio (nDevice, 1, g_nVolume);
    wgmSetVolume_CDAudio (nDevice, 2, g_nVolume);
    wgmSetVolume_CDAudio (nDevice, 3, g_nVolume);
  default:
    if (pCDTrack->nTracks <= g_nIndex)
      g_nIndex = pCDTrack->nTracks - 1;
    if (nStatus != CDROM_AUDIO_PLAY && g_nIndex < 0)
      g_nIndex = 0;
    if (0 <= g_nIndex)
    {
      if (nStatus == CDROM_AUDIO_PAUSED)
        wgmResume_CDAudio (nDevice); 
      else
        wgmPlay_CDAudio (nDevice, pCDTrack->pIndex [g_nIndex], 
          pCDTrack->pIndex [pCDTrack->nTracks]);
    }
    break;
  }

  wgmGetCurrentTrack_CDAudio (nDevice, pCDTrack);
  OutCDTrack (nDevice, pCDTrack);

  wgmFreeCDTrack_CDAudio (pCDTrack);

  wgmCloseDevice_CDAudio (nDevice);

  exit (0);
}

///////////////////////////////////////////////////////////////////////////
//>>> arginit :
///////////////////////////////////////////////////////////////////////////
static void arginit (int argc, char *argv [])
{
  char *pCmd;
  int i;

  if ((pCmd = (char *) strrchr (argv [0], '/')) != NULL)
    pCmd++;
  else
    pCmd = argv [0];
  strcpy (g_cCommand, pCmd);

  for (i=1; i<argc; i++)
  {
    if ((strcmp (argv [i], "-restart") == 0)
      || (strcmp (argv [i], "-r") == 0))
    {
      g_nCommand = ID_CDAUDIO_RESTART;
      break;
    }else if ((strcmp (argv [i], "-stop") == 0) 
      || (strcmp (argv [i], "-s") == 0)){
      g_nCommand = ID_CDAUDIO_STOP;
      break;
    }else if ((strcmp (argv [i], "-pause") == 0)
      || (strcmp (argv [i], "-p") == 0)){
      g_nCommand = ID_CDAUDIO_PAUSE;
      break;
    }else if ((strcmp (argv [i], "-resume") == 0)
      || (strcmp (argv [i], "-m") == 0)){
      g_nCommand = ID_CDAUDIO_RESUME;
      break;
    }else if ((strcmp (argv [i], "-eject") == 0)
      || (strcmp (argv [i], "-e") == 0)){
      g_nCommand = ID_CDAUDIO_EJECT;
      break;
    }else if ((strcmp (argv [i], "-close") == 0)
      || (strcmp (argv [i], "-c") == 0)){
      g_nCommand = ID_CDAUDIO_CLOSE;
      break;
    }else if ((strcmp (argv [i], "-lock") == 0)
      || (strcmp (argv [i], "-l") == 0)){
      g_nCommand = ID_CDAUDIO_LOCK;
      break;
    }else if ((strcmp (argv [i], "-unlock") == 0)
      || (strcmp (argv [i], "-u") == 0)){
      g_nCommand = ID_CDAUDIO_UNLOCK;
      break;
    }else if ((strcmp (argv [i], "-reset") == 0)
      || (strcmp (argv [i], "-t") == 0)){
      g_nCommand = ID_CDAUDIO_RESET;
      break;
    }else if ((strcmp (argv [i], "-vol") == 0)
      || (strcmp (argv [i], "-v") == 0)){
      g_nCommand = ID_CDAUDIO_VOLUME;
      if (i < argc - 1){
        if (0<atoi (argv [i+1]) && atoi (argv [i+1])<256)
          g_nVolume = atoi (argv [i+1]);
        else
          g_nCommand = 0;
        i++;
      }else{
        g_nCommand = 0;
      }
    }else if ((strcmp (argv [i], "-info") == 0)
      || (strcmp (argv [i], "-i") == 0)){
      g_nCommand = ID_CDAUDIO_CATALOGID;
      break;
    }else if ((strcmp (argv [i], "-help") == 0)
      || (strcmp (argv [i], "-h") == 0)){
      usage (TRUE);
      exit (0);
    }else{
      g_nIndex = abs (atoi (argv [i]));
      if (g_nIndex == 0 && strcmp (argv [i], "0"))
      {
        g_nIndex = 0;
        printf ("Warning: Track value error(%s). ", argv [i]);
        printf ("Set default value [start track 1]\n");
      }
      if (0 < g_nIndex)
        g_nIndex--;
    }
  }
}

///////////////////////////////////////////////////////////////////////////
//>>> usage :
///////////////////////////////////////////////////////////////////////////
static void usage (int nUsage)
{
  fprintf (stderr, ">>> " APPNAME " <<<\n");
  fprintf (stderr, "- " APPVERSION " " APPDATE " -\n");
  fprintf (stderr, "- Copyright (C) " APPPRODUCT " -\n");
  fprintf (stderr, "\n");
  fprintf (stderr, " Wingman Audio CD Player (Command line version) ... \n");
  fprintf (stderr, "\n");
  if (nUsage == FALSE)
    return;
  fprintf (stderr, " Usage: %s [-restart] [-stop] [-pause]", g_cCommand);
  fprintf (stderr, " [-resume] [-eject] [-close]\n ");
  fprintf (stderr, 
    "        [-vol <value>] [-info] [-help] track\n");
  fprintf (stderr, " -restart(or -r)     ... Play first track\n");
  fprintf (stderr, " -stop(or -s)        ... Stop cdplayer\n");
  fprintf (stderr, " -pause(or -p)       ... Pause cdplayer\n");
  fprintf (stderr, " -resume(or -m)      ... Resume cdplayer\n");
  fprintf (stderr, " -eject(or -e)       ... Eject cdrom\n");
  fprintf (stderr, " -close(or -c)       ... Close cdrom & restart\n");
  fprintf (stderr, " -lock(or -l)        ... Lock CDROM eject switch\n");
  fprintf (stderr, " -unlock(or -u)      ... Unlock CDROM eject switch\n");
  fprintf (stderr, " -reset(or -t)       ... Hard-Reset CDROM\n");
  fprintf (stderr, " -vol(or -v) <value> ... Set volume value (0-255)\n");
  fprintf (stderr, " -info(or -i)        ... Print CD Info(catalog id)\n");
  fprintf (stderr, " -help(or -h)        ... This message\n");
  fprintf (stderr, "  track              ... Play track number");
  fprintf (stderr, "(default: stopped->1, playing->resume)\n");

}

///////////////////////////////////////////////////////////////////////////
//>>> OutCDTrack :
///////////////////////////////////////////////////////////////////////////
static int OutCDTrack (int nDevice, wgmCDTrack *pCDTrack)
{
  int i, nCurTrack;
  char cCatalogID [15];

  if (pCDTrack == NULL)
    return FALSE;

  if (g_nCommand == ID_CDAUDIO_CATALOGID)
  {
    if (wgmGetCatalogID_CDAudio (nDevice, cCatalogID) == TRUE)
    {
      if (strncmp (cCatalogID, "       ", 13))
        printf (" Catalog ID: %s\n\n", cCatalogID);
      else
        printf (" Catalog ID: None\n\n");
    }else{
      printf (" Catalog ID: Error\n\n");
    }
  }

  for (i=0; i<pCDTrack->nTracks; i++)
  {
    printf (" Track%02d: %02d'%02d\"%02d %s\n", i+1,
      ((pCDTrack->pIndex [i+1]-pCDTrack->pIndex [i])/75)/60,
      ((pCDTrack->pIndex [i+1]-pCDTrack->pIndex [i])/75)%60,
       (pCDTrack->pIndex [i+1]-pCDTrack->pIndex [i])%75*100/75,
       (pCDTrack->pAudio [i] == TRUE? "Audio": "Data"));
  }
  printf (" Total  : %02d'%02d\"%02d\n\n",
    (pCDTrack->pIndex [i]/75)/60,
    (pCDTrack->pIndex [i]/75)%60,
     pCDTrack->pIndex [i]%75);

  nCurTrack = pCDTrack->nCurrentTrack - 1;
  if (nCurTrack <= 0)
    nCurTrack = 0;
  printf (" Current Track: %02d %c%02d'%02d\"%02d(%02d'%02d\"%02d)\n",
     pCDTrack->nCurrentTrack,
    (pCDTrack->nCurrentFrame - pCDTrack->pIndex [nCurTrack])<0? '-': ' ',
    (abs (pCDTrack->nCurrentFrame - pCDTrack->pIndex [nCurTrack])/75)/60,
    (abs (pCDTrack->nCurrentFrame - pCDTrack->pIndex [nCurTrack])/75)%60,
     abs (pCDTrack->nCurrentFrame - pCDTrack->pIndex [nCurTrack])%75*100/75,
    (pCDTrack->nCurrentFrame/75)/60,
    (pCDTrack->nCurrentFrame/75)%60,
     pCDTrack->nCurrentFrame%75*100/75);

  switch (pCDTrack->nCurrentStatus){
  case CDROM_AUDIO_INVALID:
    printf (" Audio status : %s\n", "INVALID"); break;
  case CDROM_AUDIO_PLAY:
    printf (" Audio status : %s\n", "PLAYING..."); break;
  case CDROM_AUDIO_PAUSED:
    printf (" Audio status : %s\n", "PAUSED"); break;
  case CDROM_AUDIO_COMPLETED:
    printf (" Audio status : %s\n", "COMPLETED"); break;
  case CDROM_AUDIO_ERROR:
    printf (" Audio status : %s\n", "ERROR"); break;
  case CDROM_AUDIO_NO_STATUS:
    printf (" Audio status : %s\n", "NO STATUS"); break;
  default:
    printf (" Device error : %d\n", pCDTrack->nCurrentStatus); break;
  }
  printf (" CDROM Volume : %d, %d, %d, %d\n", 
    wgmGetVolume_CDAudio (nDevice, 0), wgmGetVolume_CDAudio (nDevice, 1), 
    wgmGetVolume_CDAudio (nDevice, 2), wgmGetVolume_CDAudio (nDevice, 3));
  printf ("\n");

  return TRUE;
}

